#ifndef _pc_geom_euler_rotation_h_
#define _pc_geom_euler_rotation_h_

/*
 *  EulerRotation.h
 *  PcGeom
 *
 *  Created by Patrick Coleman on 6/11/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

const double HALF_PI = 1.570796326794897;

#include <PcMath/Matrix4.h>

#include "Rotation.h"

namespace PcGeom {

using PcMath::Matrix4;

enum RotationOrder {
	XYZ,
	XZY,
	YXZ,
	YZX,
	ZXY,
	ZYX
};

class EulerRotation: public Rotation {

public:
	EulerRotation(double rx, double ry, double rz, RotationOrder order = XYZ, bool local = true);
	EulerRotation(const Matrix4& matrix, RotationOrder order = XYZ, bool local = true);
	
	Matrix4 matrix() const;
	Point3 operator()(const Point3& p) const;
	Vector3 operator()(const Vector3& p) const;

public:
	double rx;
	double ry;
	double rz;
	RotationOrder order;
	bool local;

};

}

#endif
