/*
 *  PiecewiseSmoothCurve.cpp
 *  PcGeom
 *
 *  Created by Patrick Coleman on 7/27/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include "PiecewiseSmoothCurve.h"
#include "functions.h"

namespace PcGeom {

PiecewiseSmoothFloatCurve::PiecewiseSmoothFloatCurve(const vector<double>& knots, 
													 const vector<float>& values) :
		knots(knots), values(values) {
	
}

float PiecewiseSmoothFloatCurve::value(double t) const {
	
	if (t <= knots[0]) {
		return values[0];
	} else if (t >= knots[knots.size() - 1]) {
		return values[knots.size() -1];
	} else {
		int pi = 0;
		while (knots[pi + 1] < t) {
			pi += 1;
		}
		int ni = pi + 1;
		double u = (t - knots[pi]) / (double)(knots[ni] - knots[pi]);
		return bezierEval(values[pi], values[pi], values[ni], values[ni], u);
	}
}

PiecewiseSmoothVector3Curve::PiecewiseSmoothVector3Curve(const vector<double>& knots, 
														 const vector<Vector3>& values) :
		knots(knots), values(values) {
	
}

Vector3 PiecewiseSmoothVector3Curve::value(double t) const {
	
	if (t <= knots[0]) {
		return values[0];
	} else if (t >= knots[knots.size() - 1]) {
		return values[knots.size() -1];
	} else {
		int pi = 0;
		while (knots[pi + 1] < t) {
			pi += 1;
		}
		int ni = pi + 1;
		double u = (t - knots[pi]) / (double)(knots[ni] - knots[pi]);
		return bezierEval(values[pi], values[pi], values[ni], values[ni], u);
	}
}

PiecewiseSmoothPoint3Curve::PiecewiseSmoothPoint3Curve(const vector<double>& knots, 
														 const vector<Point3>& values) :
		knots(knots), values(values) {
}

Point3 PiecewiseSmoothPoint3Curve::value(double t) const {
	
	if (t <= knots[0]) {
		return values[0];
	} else if (t >= knots[knots.size() - 1]) {
		return values[values.size() -1];
	} else {
		int pi = 0;
		while (knots[pi + 1] < t) {
			pi += 1;
		}
		int ni = pi + 1;
		double u = (t - knots[pi]) / (double)(knots[ni] - knots[pi]);
		return bezierEval(values[pi], values[pi], values[ni], values[ni], u);
	}
}

PiecewiseSmoothQRotationCurve::PiecewiseSmoothQRotationCurve(const vector<double>& knots, 
														 const vector<QRotation>& values) :
		knots(knots), values(values) {
	
}

QRotation PiecewiseSmoothQRotationCurve::value(double t) const {
	
	if (t <= knots[0]) {
		return values[0];
	} else if (t >= knots[knots.size() - 1]) {
		return values[knots.size() -1];
	} else {
		int pi = 0;
		while (knots[pi + 1] < t) {
			pi += 1;
		}
		int ni = pi + 1;
		double u = (t - knots[pi]) / (double)(knots[ni] - knots[pi]);
		return bezierEval(values[pi], values[pi], values[ni], values[ni], u);
	}
}
	
}