/*
 *  Point3.cpp
 *  PcGeom
 *
 *  Created by Patrick Coleman on 5/17/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include "Point3.h"

namespace PcGeom {

Point3::Point3 (double x, double y, double z) {
	this->x = x;
	this->y = y;
	this->z = z;
}

Point3 Point3::interpolate(const Point3& rhs, double t) const {
	return Point3 (x + t * (rhs.x - x),
				   y + t * (rhs.y - y),
				   z + t * (rhs.z - z));
}

Point3 Point3::operator+(const Vector3& rhs) const {
	return Point3(x + rhs.x, y + rhs.y, z + rhs.z);
}

Point3 Point3::operator+(const Point3& rhs) const {
	return Point3(x + rhs.x, y + rhs.y, z + rhs.z);
}

Vector3 Point3::operator-(const Point3& rhs) const {
	return Vector3(x - rhs.x, y - rhs.y, z - rhs.z);
}

double Point3::distance(const Point3& other) const {
	return (other - (*this)).length();
}

double Point3::distanceSquared(const Point3& other) const {
	return (other - (*this)).lengthSquared();
}

Vector3 Point3::asVector3() const {
	return Vector3(x, y, z);
}

// ************************ ops *****************************************

Point3 interpolate(const Point3& lhs, const Point3& rhs, double t) {
	return lhs.interpolate(rhs, t);
}

Point3 operator*(double s, const Point3& p) {
	return Point3(s * p.x, s * p.y, s* p.z);
}

Point3 operator*(const Matrix4& mat, const Point3& p) {
	Point3 newP(
		mat[0][0] * p.x + mat[0][1] * p.y + mat[0][2] * p.z + mat[0][3],
		mat[1][0] * p.x + mat[1][1] * p.y + mat[1][2] * p.z + mat[1][3],
		mat[2][0] * p.x + mat[2][1] * p.y + mat[2][2] * p.z + mat[2][3]
	);
	double w = mat[3][0] * p.x + mat[3][1] * p.y + mat[3][2] * p.z + mat[3][3];
	if (w != 1.0) {
		newP.x /= w;
		newP.y /= w;
		newP.z /= w;
	}
	
	return newP;
}

ostream& operator<<(ostream& output, const Point3& p) {
	output << "[" <<  p.x << ", " <<  p.y << ", " << p.z <<"]";
    return output;
}

}
