/*
 *  Point3.h
 *  PcGeom
 *
 *  Created by Patrick Coleman on 5/17/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#ifndef _pc_geom_point_3_h_
#define _pc_geom_point_3_h_

#include <ostream>

#include <PcMath/Vector3.h>
#include <PcMath/Matrix4.h>

namespace PcGeom {

using std::ostream;

using PcMath::Vector3;
using PcMath::Matrix4;

class Point3 {

public:
	Point3(double x = 0.0, double y = 0.0, double z = 0.0);
	
	Point3 interpolate(const Point3& rhs, double t) const;

	Point3 operator+(const Vector3& rhs) const;
	Point3 operator+(const Point3& rhs) const; // needed for legible barycentric comb expressions
	Vector3 operator-(const Point3& rhs) const;
	
	double distance(const Point3& other) const;
	double distanceSquared(const Point3& other) const;
	
	Vector3 asVector3() const;

public:
	double x;
	double y;
	double z;
};

// ************************ ops *****************************************

Point3 interpolate(const Point3& lhs, const Point3& rhs, double t);

Point3 operator*(double s, const Point3& p);

Point3 operator*(const Matrix4& mat, const Point3& p);

ostream& operator<<(ostream& output, const Point3& p);

}

#endif