#ifndef _pc_geom_q_rotation_h_
#define _pc_geom_q_rotation_h_


/*
 *  QRotation.h
 *  PcGeom
 *
 *  Created by Patrick Coleman on 6/11/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include <ostream>
#include <vector>
#include <PcMath/Vector3.h>
#include <PcMath/Matrix4.h>
#include <PcMath/Quaternion.h>

#include "Rotation.h"
#include "EulerRotation.h"

namespace PcGeom {

using std::ostream;
using std::vector;
using PcMath::Vector3;
using PcMath::Matrix4;
using PcMath::Quaternion;

/**
	rotations represented using unit quaternions
*/
class QRotation: public Rotation {

public:
	
	QRotation();
	
	/**
		expects normalized axis
		expects angle in radians
	*/
	QRotation(const Vector3& axis, double angle);
	QRotation(const Quaternion& q);
	
	Matrix4 matrix() const;
	Vector3 operator()(const Vector3& v) const;
	Point3 operator()(const Point3& p) const;
	
	QRotation operator+(const QRotation& rhs) const;

	/*
		returns the rotation such that
		rot by rhs, then rot by result = this
	*/
	QRotation operator-(const QRotation& rhs) const;
	
	EulerRotation eulerRotation(RotationOrder order = XYZ, bool local = true) const; 
	
	QRotation interpolate(const QRotation& rhs, double t, bool closest = true) const;

	static QRotation getAligningRotation(const Vector3& from, const Vector3& to, bool normalized = false);
	static QRotation getAligningRotation(const Vector3& from1, const Vector3& from2,
										 const Vector3& to1, const Vector3& to2, bool normalized = false);
	static QRotation getAligningRotation(const Vector3& from1, const Vector3& from2, const Vector3& from3,
										 const Vector3& to1, const Vector3& to2, const Vector3& to3, 
										 bool normalized = false);
	static QRotation getAligningRotation(const vector<Vector3>& from, const vector<Vector3>& to, bool normalized = false);
	

public:
	Quaternion q;

};


// ************************ ops *****************************************

QRotation interpolate(const QRotation& lhs, const QRotation& rhs, double t);


ostream& operator<<(ostream& output, const QRotation& p);

}

#endif