/*
 *  Rotation.cpp
 *  PcGeom
 *
 *  Created by Patrick Coleman on 6/11/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include <math.h>
#include <iostream>

#include "Rotation.h"
#include "QRotation.h"

namespace PcGeom {

/**
	Numerically robust matrix extraction from Hansen Section 16.1
	adapted from my python libraries
*/
QRotation Rotation::qRotation() const {
		
		Matrix4 mat = matrix();
		double t = mat.trace() - mat[3][3]; // 3x3 rotation submatrix trace
		double q0 = 0.5 * sqrt(t + 1.0);
		
		if (t > 0.0) {
			double inv4Q0 = 1.0 / (4.0 * q0);
			double q1 = inv4Q0 * (mat[2][1] - mat[1][2]);
			double q2 = inv4Q0 * (mat[0][2] - mat[2][0]);
			double q3 = inv4Q0 * (mat[1][0] - mat[0][1]);
			return QRotation(Quaternion(q0, Vector3(q1, q2, q3)));
		} else {
			double q1 = 0.5 * sqrt(1.0 + mat[0][0] - mat[1][1] - mat[2][2]);
			double q2 = 0.5 * sqrt(1.0 + mat[1][1] - mat[2][2] - mat[0][0]);
			double q3 = 0.5 * sqrt(1.0 + mat[2][2] - mat[0][0] - mat[1][1]);
			if (q1 > q2 and q1 > q3) {
				double inv4Q1 = 1.0 / (4.0 * q1);
				q0 = inv4Q1 * (mat[2][1] - mat[1][2]);
				q2 = inv4Q1 * (mat[1][0] + mat[0][1]);
				q3 = inv4Q1 * (mat[0][2] + mat[2][0]);
				return QRotation(Quaternion(q0, Vector3(q1, q2, q3)));
			} else if (q2 > q1 and q2 > q3) {
				double inv4Q2 = 1.0 / (4.0 * q2);
				q0 = inv4Q2 * (mat[0][2] - mat[2][0]);
				q1 = inv4Q2 * (mat[0][1] + mat[1][0]);
				q3 = inv4Q2 * (mat[1][2] + mat[2][1]);
				return QRotation(Quaternion(q0, Vector3(q1, q2, q3)));
			} else { // q3 >= q1 and q3 >= q2
				double inv4Q3 = 1.0 / (4.0 * q3);
				q0 = inv4Q3 * (mat[1][0] - mat[0][1]) ;
				q1 = inv4Q3 * (mat[2][0] + mat[0][2]);
				q2 = inv4Q3 * (mat[1][2] + mat[2][1]);
				return QRotation(Quaternion(q0, Vector3(q1, q2, q3)));
			}
		}
}

}

