#ifndef _pc_geom_functions_h_
#define _pc_geom_functions_h_

/*
 *  functions.h
 *  PcGeom
 *
 *  Created by Patrick Coleman on 7/7/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include <vector>
#include "Point3.h"
#include "QRotation.h"
#include "BoundingBox.h"

namespace PcGeom {

using std::vector;

bool colocated(const vector<Point3>& samples, double tol = 0.001);

bool colocated(const vector<Vector3>& samples, double tol = 0.001);

/**
	returns indices to knots bounding t
	they might be equal if t is equal to a knot
*/
void getBoundingIndices(const vector<float>& knots, float t, int& priorIndex, int& nextIndex);


float bezierEval(float v1, float v2, float v3, float v4, float t);

Vector3 bezierEval(const Vector3& v1, const Vector3& v2, const Vector3& v3, 
				   const Vector3& v4, float t);

Point3 bezierEval(const Point3& p1, const Point3& p2, const Point3& p3, 
				  const Point3& p4, float t);

/**
	recursive spherical linear interp in S^3, a la shoemake
*/
QRotation bezierEval(const QRotation& q1, const QRotation& q2, const QRotation& q3,
					 const QRotation& q4, float t);

}

#endif