/*
 *  Matrix.cpp
 *  PcMath
 *
 *  Created by Patrick Coleman on 7/8/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include "Matrix.h"

namespace PcMath {

Matrix::Matrix(int numRows, int numCols) {
	initializeData(numRows, numCols);
}

Matrix::Matrix(const vector<vector<double> >& data) :
	data(data) {
}

vector<double>& Matrix::operator[](int i) {
	return data[i];
}

const vector<double>& Matrix::operator[](int i) const {
	return data[i];
}
	
void Matrix::setToIdentity() {
	for (int i = 0; i < rowSize(); i++) {
	for (int j = 0; j < columnSize(); j++) {
		if (i== j) {
			data[i][j] = 1.0;
		} else {
			data[i][j] = 0.0;
		}
	}
	}
}

void Matrix::clear() {
	for (int i = 0; i < rowSize(); i++) {
	for (int j = 0; j < columnSize(); j++) {
		data[i][j] = 0.0;
	}
	}
}
	
int Matrix::rowSize() const {
	return data.size();
}

int Matrix::columnSize() const {
	if (rowSize() == 0) {
		return 0;
	} else {
		return data[0].size();
	}
}

void Matrix::initializeData(int numRows, int numCols) {
	data.clear();
	for (int i = 0; i < numRows; i++) {
		vector<double> row;
		for (int j = 0; j < numCols; j++) {
			row.push_back(0.0);
		}	
		data.push_back(row);
	}
}

}
