/*
 *  Matrix4.cpp
 *  PcMath
 *
 *  Created by Patrick Coleman on 6/11/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include "Matrix4.h"

namespace PcMath {
	
Matrix4::Matrix4() {
	initializeData();
}

vector<double>& Matrix4::operator[](int i) {
	return data[i];
}

const vector<double>& Matrix4::operator[](int i) const {
	return data[i];
}

Matrix4 Matrix4::operator*(const Matrix4& rhs) const {
	Matrix4 mat;
	for (int i = 0; i < 4; i++) {
	for (int j = 0; j < 4; j++) {
		mat[i][j] = 
			data[i][0] * rhs[0][j] + 
			data[i][1] * rhs[1][j] + 
			data[i][2] * rhs[2][j] + 
			data[i][3] * rhs[3][j];
	}
	}
	return mat;
}

Vector3 Matrix4::operator*(const Vector3& rhs) const {
	return Vector3(
		data[0][0] * rhs.x + data[0][1] * rhs.y + data[0][2] * rhs.z,
		data[1][0] * rhs.x + data[1][1] * rhs.y + data[1][2] * rhs.z,
		data[2][0] * rhs.x + data[2][1] * rhs.y + data[2][2] * rhs.z
	);
}

double Matrix4::trace() const {
	return data[0][0] + data[1][1] + data[2][2] + data[3][3];
}

double Matrix4::det() const {
	return (
		data[0][3] * data[1][2] * data[2][1] * data[3][0]
			-data[0][2] * data[1][3] * data[2][1] * data[3][0]
			-data[0][3] * data[1][1] * data[2][2] * data[3][0]
			+data[0][1] * data[1][3] * data[2][2] * data[3][0] +
		data[0][2] * data[1][1] * data[2][3] * data[3][0]
			-data[0][1] * data[1][2] * data[2][3] * data[3][0]
			-data[0][3] * data[1][2] * data[2][0] * data[3][1]
			+data[0][2] * data[1][3] * data[2][0] * data[3][1] +
		data[0][3] * data[1][0] * data[2][2] * data[3][1]
			-data[0][0] * data[1][3] * data[2][2] * data[3][1]
			-data[0][2] * data[1][0] * data[2][3] * data[3][1]
			+data[0][0] * data[1][2] * data[2][3] * data[3][1] +
		data[0][3] * data[1][1] * data[2][0] * data[3][2]
			-data[0][1] * data[1][3] * data[2][0] * data[3][2]
			-data[0][3] * data[1][0] * data[2][1] * data[3][2]
			+data[0][0] * data[1][3] * data[2][1] * data[3][2] +
		data[0][1] * data[1][0] * data[2][3] * data[3][2]
			-data[0][0] * data[1][1] * data[2][3] * data[3][2]
			-data[0][2] * data[1][1] * data[2][0] * data[3][3]
			+data[0][1] * data[1][2] * data[2][0] * data[3][3] +
		data[0][2] * data[1][0] * data[2][1] * data[3][3]
			-data[0][0] * data[1][2] * data[2][1] * data[3][3]
			-data[0][1] * data[1][0] * data[2][2] * data[3][3]
			+data[0][0] * data[1][1] * data[2][2] * data[3][3]
	);
}

Matrix4 Matrix4::inverse() const {
	double s = 1.0 / det();
	
	Matrix4 m;
	m[0][0] = data[1][2] * data[2][3] * data[3][1] - data[1][3] * data[2][2] * data[3][1] + data[1][3] * data[2][1] * data[3][2] - data[1][1] * data[2][3] * data[3][2] - data[1][2] * data[2][1] * data[3][3] + data[1][1] * data[2][2] * data[3][3];
	m[0][1] = data[0][3] * data[2][2] * data[3][1] - data[0][2] * data[2][3] * data[3][1] - data[0][3] * data[2][1] * data[3][2] + data[0][1] * data[2][3] * data[3][2] + data[0][2] * data[2][1] * data[3][3] - data[0][1] * data[2][2] * data[3][3];
	m[0][2] = data[0][2] * data[1][3] * data[3][1] - data[0][3] * data[1][2] * data[3][1] + data[0][3] * data[1][1] * data[3][2] - data[0][1] * data[1][3] * data[3][2] - data[0][2] * data[1][1] * data[3][3] + data[0][1] * data[1][2] * data[3][3];
	m[0][3] = data[0][3] * data[1][2] * data[2][1] - data[0][2] * data[1][3] * data[2][1] - data[0][3] * data[1][1] * data[2][2] + data[0][1] * data[1][3] * data[2][2] + data[0][2] * data[1][1] * data[2][3] - data[0][1] * data[1][2] * data[2][3];
	m[1][0] = data[1][3] * data[2][2] * data[3][0] - data[1][2] * data[2][3] * data[3][0] - data[1][3] * data[2][0] * data[3][2] + data[1][0] * data[2][3] * data[3][2] + data[1][2] * data[2][0] * data[3][3] - data[1][0] * data[2][2] * data[3][3];
	m[1][1] = data[0][2] * data[2][3] * data[3][0] - data[0][3] * data[2][2] * data[3][0] + data[0][3] * data[2][0] * data[3][2] - data[0][0] * data[2][3] * data[3][2] - data[0][2] * data[2][0] * data[3][3] + data[0][0] * data[2][2] * data[3][3];
	m[1][2] = data[0][3] * data[1][2] * data[3][0] - data[0][2] * data[1][3] * data[3][0] - data[0][3] * data[1][0] * data[3][2] + data[0][0] * data[1][3] * data[3][2] + data[0][2] * data[1][0] * data[3][3] - data[0][0] * data[1][2] * data[3][3];
	m[1][3] = data[0][2] * data[1][3] * data[2][0] - data[0][3] * data[1][2] * data[2][0] + data[0][3] * data[1][0] * data[2][2] - data[0][0] * data[1][3] * data[2][2] - data[0][2] * data[1][0] * data[2][3] + data[0][0] * data[1][2] * data[2][3];
	m[2][0] = data[1][1] * data[2][3] * data[3][0] - data[1][3] * data[2][1] * data[3][0] + data[1][3] * data[2][0] * data[3][1] - data[1][0] * data[2][3] * data[3][1] - data[1][1] * data[2][0] * data[3][3] + data[1][0] * data[2][1] * data[3][3];
	m[2][1] = data[0][3] * data[2][1] * data[3][0] - data[0][1] * data[2][3] * data[3][0] - data[0][3] * data[2][0] * data[3][1] + data[0][0] * data[2][3] * data[3][1] + data[0][1] * data[2][0] * data[3][3] - data[0][0] * data[2][1] * data[3][3];
	m[2][2] = data[0][1] * data[1][3] * data[3][0] - data[0][3] * data[1][1] * data[3][0] + data[0][3] * data[1][0] * data[3][1] - data[0][0] * data[1][3] * data[3][1] - data[0][1] * data[1][0] * data[3][3] + data[0][0] * data[1][1] * data[3][3];
	m[2][3] = data[0][3] * data[1][1] * data[2][0] - data[0][1] * data[1][3] * data[2][0] - data[0][3] * data[1][0] * data[2][1] + data[0][0] * data[1][3] * data[2][1] + data[0][1] * data[1][0] * data[2][3] - data[0][0] * data[1][1] * data[2][3];
	m[3][0] = data[1][2] * data[2][1] * data[3][0] - data[1][1] * data[2][2] * data[3][0] - data[1][2] * data[2][0] * data[3][1] + data[1][0] * data[2][2] * data[3][1] + data[1][1] * data[2][0] * data[3][2] - data[1][0] * data[2][1] * data[3][2];
	m[3][1] = data[0][1] * data[2][2] * data[3][0] - data[0][2] * data[2][1] * data[3][0] + data[0][2] * data[2][0] * data[3][1] - data[0][0] * data[2][2] * data[3][1] - data[0][1] * data[2][0] * data[3][2] + data[0][0] * data[2][1] * data[3][2];
	m[3][2] = data[0][2] * data[1][1] * data[3][0] - data[0][1] * data[1][2] * data[3][0] - data[0][2] * data[1][0] * data[3][1] + data[0][0] * data[1][2] * data[3][1] + data[0][1] * data[1][0] * data[3][2] - data[0][0] * data[1][1] * data[3][2];
	m[3][3] = data[0][1] * data[1][2] * data[2][0] - data[0][2] * data[1][1] * data[2][0] + data[0][2] * data[1][0] * data[2][1] - data[0][0] * data[1][2] * data[2][1] - data[0][1] * data[1][0] * data[2][2] + data[0][0] * data[1][1] * data[2][2];
	return s * m;
}

void Matrix4::setToIdentity() {
	for (int i = 0; i < 4; i++) {
		for (int j = 0; j < 4; j++) {
			if (i  == j) {
				data[i][j] = 1.0;
			} else {
				data[i][j] = 0.0;
			}
		}
	}
}

void Matrix4::initializeData() {
	data.clear();
	for (int i = 0; i < 4; i++) {
		vector<double> row;
		for (int j = 0; j < 4; j++) {
			row.push_back(0.0);
		}
		data.push_back(row);
	}
	setToIdentity();
}

// OPS

Matrix4 operator*(double s, const Matrix4& mat) {

	Matrix4 res;
	for (int i = 0; i < 4; i++) {
	for (int j = 0; j < 4; j++) {
		res[i][j] = s * mat[i][j];
	}
	
	}
	return res;
}
	
}