/*
 *  Quaternion.cpp
 *  PcMath
 *
 *  Created by Patrick Coleman on 6/11/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include "Quaternion.h"

namespace PcMath {


Quaternion::Quaternion(double a, double b, double c, double d) {
	this->a = a;
	this->b = b;
	this->c = c;
	this->d = d;
}

Quaternion::Quaternion(double s, const Vector3& v) {
	this->a = s;
	this->b = v.x;
	this->c = v.y;
	this->d = v.z;
}

Quaternion Quaternion::operator*(const Quaternion& rhs) const {
	return Quaternion(
		a * rhs.a - b * rhs.b - c * rhs.c - d * rhs.d, 
		a * rhs.b + b * rhs.a + c * rhs.d - d * rhs.c,
		a * rhs.c - b * rhs.d + c * rhs.a + d * rhs.b,
		a * rhs.d + b * rhs.c - c * rhs.b + d * rhs.a
		);
}

Quaternion Quaternion::operator+(const Quaternion& rhs) const {
	return Quaternion(
		a + rhs.a, 
		b + rhs.b,
		c + rhs.c,
		d + rhs.d
		);
}

Quaternion Quaternion::conjugate() const {
	return Quaternion(a, -b, -c, -d);
}

double Quaternion::dot(const Quaternion& rhs) const {
	return (a * rhs.a + b * rhs.b + c * rhs.c + d * rhs.d);
}

Quaternion Quaternion::operator-() const {
	return Quaternion(-a, -b, -c, -d);
}

// ************** ops ***************

Quaternion operator*(double s, const Quaternion& rhs) {
	return Quaternion(s * rhs.a, s * rhs.b, s * rhs.c, s * rhs.d);
}

ostream& operator<<(ostream& output, const Quaternion& q) {
	output << "[" <<  q.a << ", " <<  q.b << ", " <<  q.c << ", " << q.d <<"]";
    return output;
}

}
