/*
 *  Vector3.cpp
 *  pcMath
 *
 *  Created by Patrick Coleman on 5/16/09.
 *  Copyright 2009 patrick. All rights reserved.
 *
 */

#include <math.h>

#include "Vector3.h"

namespace PcMath {

Vector3::Vector3 (double x, double y, double z) {
	this->x = x;
	this->y = y;
	this->z = z;
}

Vector3& Vector3::operator=(const Vector3& rhs) {
	if (this != &rhs) {
		x = rhs.x;
		y = rhs.y;
		z = rhs.z;
	}
	return *this;
}

Vector3 Vector3::operator+(const Vector3& rhs) const {
	return Vector3(x + rhs.x, y + rhs.y, z + rhs.z);
}

Vector3 Vector3::operator-(const Vector3& rhs) const {
	return Vector3(x - rhs.x, y - rhs.y, z - rhs.z);
}

Vector3& Vector3::operator*=(double s) {
	x *= s;
	y *= s;
	z *= s;
	return *this;
}

double Vector3::operator*(const Vector3& rhs) const {
	return (x * rhs.x + y * rhs.y + z * rhs.z);
}

Vector3 Vector3::operator^(const Vector3& rhs) const {
	return Vector3(y * rhs.z - z * rhs.y,
				   z * rhs.x - x * rhs.z,
				   x * rhs.y - y * rhs.x);
}

double Vector3::length() const {
	return sqrt((*this) * (*this));
}

double Vector3::lengthSquared() const {
	return (*this) * (*this);
}

Vector3 Vector3::normalize() const {
	double invLen = length();
	if (invLen > 0.000001) {
		invLen = 1.0 / invLen;
	}
	return invLen * (*this);
}

double Vector3::angle(const Vector3& rhs) const {
	double arg = ((*this) * rhs) / (length() * rhs.length());
			
	// avoid numerical error breaking acos
	if (arg > 1.0) {
		arg = 1.0;
	}
			
	return acos(arg);
}

// ************************** ops *****************************

Vector3 operator*(double s, const Vector3& rhs) {
	return Vector3(s * rhs.x, s * rhs.y, s * rhs.z);
}


}