#ifndef _pc_motion_contact_detector_h_
#define _pc_motion_contact_detector_h_

/*
 *  ContactDetector.h
 *  PcMotion
 *
 *  Created by Patrick Coleman on 8/18/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include <vector>
#include <string>

#include "Contact.h"
#include "SampledMotion.h"
#include "RigidHierarchy.h"

namespace PcMotion {

using std::vector;
using std::string;

class ContactDetector {

public:
	ContactDetector(SampledMotion* motion, const RigidHierarchy& rh,
					const vector<string>& contactJoints,
					const vector<string>& mergeJoints,
					float tolerance = 0.1f, int combineWidth = 5, float dt = 0.0833333); // 120 fps
	ContactMap getContacts();
	ContactMap getMergedContacts();

private:
	SampledMotion* motion;
	RigidHierarchy rh;
	vector<string> contactJoints;
	vector<string> mergeJoints;
	float tolerance;
	int combineWidth;
	float dt;
	
	bool contactsCached;
	bool mergedContactsCached;
	ContactMap contacts;
	ContactMap mergedContacts;
	
	
	void computeContacts();
	void computeMergedContacts();
};

}

#endif