#ifndef _pc_motion_contact_enforcer_h_
#define _pc_motion_contact_enforcer_h_

/*
 *  ContactEnforcer.h
 *  PcMotion
 *
 *  Created by Patrick Coleman on 8/21/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include <vector>
#include <string>

#include <PcSignal/FloatSample.h>

#include "MotionTransformation.h"
#include "Contact.h"
#include "RigidHierarchy.h"

namespace PcMotion {

using std::vector;
using std::string;

using PcSignal::FloatSample;

class ContactEnforcer: public MotionTransformation {

public: 
	ContactEnforcer(const ContactMap& contacts, const ContactMap& mergedContacts,
				    SampledMotion* referenceMotion, const RigidHierarchy& rh,
					const vector<string>& ikStartJoints, const vector<string>& ikEndJoints);

	void apply(SampledMotion* motion) const;
	
	vector<float> getVerticalOffsetSignal() const;  // returns vertical offset signal from prior call to apply
	
private:
	ContactMap contactMap;
	mutable ContactMap mergedContactMap;
	SampledMotion* referenceMotion;
	RigidHierarchy rh;
	vector<string> ikChainStartJoints;	// i.e. hips/shoulders
	vector<string> ikChainEndJoints;	// i.e. ankles/wrists
	
	mutable vector<float> verticalOffsetSignal;
	
	// big algorithm steps
	void copyIkChildrenLocal(SampledMotion* motion) const;
	void adjustVerticalMotion(SampledMotion* motion) const;
	void adjustGroundParallelMotion(SampledMotion* motion) const;
	void copyAndWarpIkChildrenTrajectories(SampledMotion* motion) const;
	void adjustIKChainSingleJoint(SampledMotion* motion) const;  // only handles one intermediate joint

	// subops for vertical adustment
	void getVerticalMergedContactSamples(SampledMotion* motion, 
									 vector< vector<FloatSample> >& samples) const;
	void mergeFloatIndicesAndOffsets(SampledMotion* motion,
									 const vector< vector<FloatSample> >& samples,
									 vector<FloatSample>& mergedSamples) const;


public:
	bool disableLocalCopy; // disables local copy of end joint children
	bool disableVerticalCorrection;
};

}

#endif