/*
 *  CurveMotion.cpp
 *  PcMotion
 *
 *  Created by Patrick Coleman on 7/27/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include "CurveMotion.h"
#include <PcGeom/PiecewiseSmoothCurve.h>

namespace PcMotion {

using PcGeom::PiecewiseSmoothPoint3Curve;
using PcGeom::PiecewiseSmoothQRotationCurve;

CurveMotion::CurveMotion(double startTime, 
						 const vector<double>& times, const vector<Pose>& poses):
	Motion(startTime, poses[0].getTransAttrs(), poses[0].getRotAttrs()) {
	
	for (int i =0; i < numTransAttrs(); i++) {
		string tAttr = transAttr(i);
		vector<Point3> values;
		for (int j = 0; j < poses.size(); j++) {
			values.push_back(poses[j].transValue(tAttr));
		}
		setTransCurve(tAttr, new PiecewiseSmoothPoint3Curve(times, values));
	}
	
	for (int i =0; i < numRotAttrs(); i++) {
		string rAttr = rotAttr(i);
		vector<QRotation> values;
		for (int j = 0; j < poses.size(); j++) {
			values.push_back(poses[j].rotValue(rAttr));
		}
		setRotCurve(rAttr, new PiecewiseSmoothQRotationCurve(times, values));
	}	
}

CurveMotion::~CurveMotion() {
	map<const string, Point3Curve*>::iterator tcIter;
	map<const string, QRotationCurve*>::iterator rcIter;
	
	for (tcIter = transCurves.begin(); tcIter != transCurves.end(); tcIter++) {
		if (tcIter->second != 0) {
			delete tcIter->second; 
		}
	}
	
	for (rcIter = rotCurves.begin(); rcIter != rotCurves.end(); rcIter++) {
		if (rcIter->second != 0) {
			delete rcIter->second; 
		}
	}
}
	
void CurveMotion::setTransCurve(const string& attribute, Point3Curve* curve) {
	map<const string, Point3Curve*>::iterator tcIter = transCurves.find(attribute);
	if (tcIter != transCurves.end()) {
		delete tcIter->second;
	}
	transCurves[attribute] = curve;
}

void CurveMotion::setRotCurve(const string& attribute, QRotationCurve* curve) {
	map<const string, QRotationCurve*>::iterator rcIter = rotCurves.find(attribute);
	if (rcIter != rotCurves.end()) {
		delete rcIter->second;
	}
	rotCurves[attribute] = curve;
}

void CurveMotion::setTransCurves(map<const string, Point3Curve*>& curves) {
	map<const string, Point3Curve*>::iterator tcIter;
	for (tcIter = curves.begin(); tcIter != curves.end(); tcIter++) {
		setTransCurve(tcIter->first, tcIter->second);
	}
}

void CurveMotion::setRotCurves(map<const string, QRotationCurve*>& curves) {
	map<const string, QRotationCurve*>::iterator rcIter;
	for (rcIter = curves.begin(); rcIter != curves.end(); rcIter++) {
		setRotCurve(rcIter->first, rcIter->second);
	}
}
	
Point3 CurveMotion::transValue(const string& attribute, double t) const {
	return transCurves[attribute]->value(t);
}

QRotation CurveMotion::rotValue(const string& attribute, double t) const {
	return rotCurves[attribute]->value(t);
}



}