#ifndef _pc_motion_curve_motion_h_
#define _pc_motion_curve_motion_h_

/*
 *  CurveMotion.h
 *  PcMotion
 *
 *  Created by Patrick Coleman on 7/27/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include <map>
#include <PcGeom/Curve.h>
#include "Motion.h"

namespace PcMotion {

using std::map;
using PcGeom::Point3Curve;
using PcGeom::QRotationCurve;

class CurveMotion: public Motion {

public:
	CurveMotion(double startTime): Motion(startTime) {}
	CurveMotion(double startTime, 
				const vector<string>& transAttrs, 
				const vector<string>& rotAttrs) : Motion(startTime, transAttrs, rotAttrs) {}
	CurveMotion(double startTime, const vector<double>& times, const vector<Pose>& poses);
	
	virtual ~CurveMotion();
	
	// curve stuff (takes ownership
	
	void setTransCurve(const string& attribute, Point3Curve* curve);
	void setRotCurve(const string& attribute, QRotationCurve* curve);
	void setTransCurves(map<const string, Point3Curve*>& curves);
	void setRotCurves(map<const string, QRotationCurve*>& curves);
	
	// for time based evaluation, takes startTime into account
	virtual Point3 transValue(const string& attribute, double t) const;
	virtual QRotation rotValue(const string& attribute, double t) const;
	
private:
	mutable map<const string, Point3Curve*> transCurves;
	mutable map<const string, QRotationCurve*> rotCurves;
	

};

}

#endif