/*
 *  Pose.cpp
 *  PcMotion
 *
 *  Created by Patrick Coleman on 6/24/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include <PcMath/Vector3.h>

#include "Pose.h"

namespace PcMotion {

using PcMath::Vector3;

Pose::Pose() : RigidHierarchicalObject() {
		
}

Pose::Pose(const vector<string>& transAttrs, const vector<string>& rotAttrs) 
	: RigidHierarchicalObject(transAttrs, rotAttrs) {
		
}

Pose::Pose(const Pose& other) :
	RigidHierarchicalObject(other),
	transValues(other.transValues),
	rotValues(other.rotValues) {
	
}

Point3 Pose::transValue(const string& attribute) const {
	return transValues[attribute];
}

QRotation Pose::rotValue(const string& attribute) const {
	return rotValues[attribute];
}

void Pose::setTransValue(const string& attribute, const Point3& value) {
	transValues[attribute] = value;
}

void Pose::setRotValue(const string& attribute, const QRotation& value) {
	rotValues[attribute] = value;
}

Pose Pose::operator-(const Pose& rhs) const {

	Pose resPose(*this);
	
	for (int i = 0; i < numTransAttrs(); i++) {
		string tAttr = transAttrs[i];
		if (rhs.hasTransAttr(tAttr)) {
			Vector3 diff = transValues[tAttr] - rhs.transValues[tAttr];
			resPose.setTransValue(tAttr, Point3(diff.x, diff.y, diff.z));
		} 
	}
	
	for (int i = 0; i < numRotAttrs(); i++) {
		string rAttr = rotAttrs[i];
		if (rhs.hasRotAttr(rAttr)) {
			QRotation diff = rotValues[rAttr] - rhs.rotValues[rAttr];
			resPose.setRotValue(rAttr, diff);
		} 
	}

	return resPose;
}

double Pose::euclideanStanceDistance(const Pose& other, const string& root) const {
	// TODO
	/*
	sum of differences betwen p - root and otherP - otherRoot
		/ numTransAttrs - 1 (not including root)
	*/
	int numPts = numTransAttrs();
	double sum = 0.0;
	Point3 rootPos = transValue(root);
	Point3 otherRootPos = other.transValue(root);
	
	for (int i = 0; i < numPts; i++) {
		if (transAttr(i) != root) {
			Vector3 rootRelativePos = transValue(transAttr(i)) - rootPos;
			Vector3 otherRelativePos = other.transValue(transAttr(i)) - otherRootPos;
			sum += (rootRelativePos - otherRelativePos).lengthSquared();
		}
	}

	return sum / (double)numPts;
}
	
}
