#ifndef _pc_motion_pose_h_
#define _pc_motion_pose_h_


/*
 *  Pose.h
 *  PcMotion
 *
 *  Created by Patrick Coleman on 6/24/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include <map>
#include <PcGeom/Point3.h>
#include <PcGeom/QRotation.h>

#include "RigidHierarchicalObject.h"
#include "RigidHierarchy.h"

namespace PcMotion {

using std::map;
using PcGeom::Point3;
using PcGeom::QRotation;

class Pose: public RigidHierarchicalObject {

public:
	Pose();
	Pose(const vector<string>& transAttrs, const vector<string>& rotAttrs);
	Pose(const Pose& other);
	virtual ~Pose() {};
	
	Point3 transValue(const string& attribute) const;
	QRotation rotValue(const string& attribute) const;
	void setTransValue(const string& attribute, const Point3& value);
	void setRotValue(const string& attribute, const QRotation& value);
	
	/*
		compute a pose that stores the parameter differences between this and rhs pose
	*/
	Pose operator-(const Pose& rhs) const;
	
	/**
		compute a stance difference of overall translation values
	*/
	double euclideanStanceDistance(const Pose& other, const string& root) const;
	
private:
	mutable map<string, Point3> transValues;
	mutable map<string, QRotation> rotValues;

};


}

#endif