/*
 *  RigidHierarchicalObject.cpp
 *  PcMotion
 *
 *  Created by Patrick Coleman on 6/17/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include "RigidHierarchicalObject.h"

namespace PcMotion {

RigidHierarchicalObject::RigidHierarchicalObject() {
	transAttrs.clear();
	rotAttrs.clear();
}

RigidHierarchicalObject::RigidHierarchicalObject(
	const vector<string>& transAttrs, 
	const vector<string>& rotAttrs) {
	
	this->transAttrs = transAttrs;
	this->rotAttrs = rotAttrs;
}

RigidHierarchicalObject::RigidHierarchicalObject(const RigidHierarchicalObject& other):
	transAttrs(other.transAttrs), rotAttrs(other.rotAttrs) {
	
}

int RigidHierarchicalObject::numTransAttrs() const {
	return transAttrs.size();
}
int RigidHierarchicalObject::numRotAttrs() const {
	return rotAttrs.size();
}

string RigidHierarchicalObject::transAttr(int i) const {
	return transAttrs[i];
}

string RigidHierarchicalObject::rotAttr(int i) const {
	return rotAttrs[i];
}

vector<string> RigidHierarchicalObject::getTransAttrs() const {
	return transAttrs;
}

vector<string> RigidHierarchicalObject::getRotAttrs() const {
	return rotAttrs;
}

void RigidHierarchicalObject::appendTransAttr(const string& attribute) {
	transAttrs.push_back(attribute);
}

void RigidHierarchicalObject::appendRotAttr(const string& attribute) {
	rotAttrs.push_back(attribute);
}

void RigidHierarchicalObject::setTransAttr(int i, const string& attribute) {
	transAttrs[i] = attribute;
}

void RigidHierarchicalObject::setRotAttr(int i, const string& attribute) {
	rotAttrs[i] = attribute;
}

void RigidHierarchicalObject::setTransAttrs(const vector<string>& attributes) {
	transAttrs = attributes;
}

void RigidHierarchicalObject::setRotAttrs(const vector<string>& attributes) {
	rotAttrs = attributes;
}
	
void RigidHierarchicalObject::clearAttributes() {
	transAttrs.clear();
	rotAttrs.clear();
}

bool RigidHierarchicalObject::hasTransAttr(const string& attr) const {
	for (int i = 0; i < transAttrs.size(); i++) {
		if (transAttrs[i] == attr) {
			return true;
		}
	}
	return false;
}

bool  RigidHierarchicalObject::hasRotAttr(const string& attr) const {
	for (int i = 0; i < rotAttrs.size(); i++) {
		if (rotAttrs[i] == attr) {
			return true;
		}
	}
	return false;
}



}