#ifndef _pc_motion_rigid_hierarchical_object_h_
#define _pc_motion_rigid_hierarchical_object_h_

/*
 *  RigidHierarchicalObject.h
 *  PcMotion
 *
 *  Created by Patrick Coleman on 6/17/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include <string>
#include <vector>

namespace PcMotion {

using std::string;
using std::vector;

/**
	abstract superclass for objects that store data tied to rigid
	hierarchies (i.e. poses, motions)
	has no sense of structure (use RigidHierarchy to represent structure)
*/
class RigidHierarchicalObject {

public:
	
	RigidHierarchicalObject();
	RigidHierarchicalObject(const vector<string>& transAttrs, const vector<string>& rotAttrs);
	RigidHierarchicalObject(const RigidHierarchicalObject& other);
	virtual ~RigidHierarchicalObject() {};
	
	int numTransAttrs() const;
	int numRotAttrs() const;
	string transAttr(int i) const;
	string rotAttr(int i) const;
	vector<string> getTransAttrs() const;
	vector<string> getRotAttrs() const;
	void appendTransAttr(const string& attribute);
	void appendRotAttr(const string& attribute);
	void setTransAttr(int i, const string& attribute);
	void setRotAttr(int i, const string& attribute);
	void setTransAttrs(const vector<string>& attributes);
	void setRotAttrs(const vector<string>& attributes);
	void clearAttributes();
	bool hasTransAttr(const string& attr) const;
	bool hasRotAttr(const string& attr) const;
	
protected:
	vector<string> transAttrs;
	vector<string> rotAttrs;
};

}

#endif