#ifndef _pc_motion_rigid_hierarchy_h_
#define _pc_motion_rigid_hierarchy_h_

/*
 *  RigidHierarchy.h
 *  PcMotion
 *
 *  Created by Patrick Coleman on 6/24/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include <vector>
#include <string>
#include <map>

#include <PcFoundation/ReferencedObject.h>
#include <PcFoundation/Reference.h>

namespace PcMotion {

using std::vector;
using std::string;
using std::map;
using PcFoundation::ReferencedObject;
using PcFoundation::Reference;

class RHNode: public ReferencedObject {

public:
	RHNode(const string& name);

	string name;
	vector<Reference<RHNode> > children;
	Reference<RHNode> parent;
};

/**
	class for representing structure of a rigid hierarchy 
*/
class RigidHierarchy {

public:
	RigidHierarchy(const string& rootName);

	string root() const;
	string parent(const string& node) const;
	string child(const string& node, int i) const;
	int numChildren(const string& node) const;
	void addChild(const string& node, const string& name);
	vector<string> allNodes() const;
	vector<string> descendants(const string& node) const;
	bool isDescendant(const string& node, const string& desc);
	RigidHierarchy* copy() const;

private:
	Reference<RHNode> rootNode;
	mutable map<string, Reference<RHNode> > nodeMap;
	
	void copyDescendants(const string& node, RigidHierarchy& other) const;
};

}

#endif