/*
 *  RigidTransformHierarchy.cpp
 *  PcMotion
 *
 *  Created by Patrick Coleman on 7/7/09.
 *  Copyright 2009 __MyCompanyName__. All rights reserved.
 *
 */

#include <PcGeom/Translation.h>

#include "RigidTransformHierarchy.h"

namespace PcMotion {

using PcGeom::Translation;

RigidTransformHierarchy::RigidTransformHierarchy(const RigidHierarchy& rh, const Pose& p) :
	hierarchy(rh), pose(p) {
	
}

void RigidTransformHierarchy::setPose(const Pose& p) {
	pose = p;
}
	
MatrixTransform RigidTransformHierarchy::getLocalTransform(const string& node) const {
	QRotation qr = pose.rotValue(node);
	Translation tr = Translation(pose.transValue(node));
	return tr + qr;  // rotate then translate, as in combined affine transform
}
	
MatrixTransform RigidTransformHierarchy::getWorldTransform(const string& node) const {

	MatrixTransform mt = getLocalTransform(node);
	string parent = hierarchy.parent(node);
	while (parent != "") {
		mt = getLocalTransform(parent) + mt; 
		parent = hierarchy.parent(parent);
	}
	return mt;
}
	
MatrixTransform RigidTransformHierarchy::getInverseWorldTransform(const string& node) const {
	return MatrixTransform(getWorldTransform(node).matrix().inverse());
}



}