#ifndef _pc_motion_rigid_transform_hierarchy_h_
#define _pc_motion_rigid_transform_hierarchy_h_

/*
 *  RigidTransformHierarchy.h
 *  PcMotion
 *
 *  Created by Patrick Coleman on 7/7/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include <PcGeom/MatrixTransform.h>

#include "RigidHierarchy.h"
#include "Pose.h"

namespace PcMotion {

using PcGeom::MatrixTransform;

/**
	combines a rigid hierarchy with transforms per node
	to allow for fast retrieval of combined transforms from pose data
*/
class RigidTransformHierarchy {

public:
	RigidTransformHierarchy(const RigidHierarchy& rh, const Pose& p);
	
	void setPose(const Pose& p);
	
	/**
		returns the local transform for the node
	*/
	MatrixTransform getLocalTransform(const string& node) const;
	
	/**
		returns the transform from the space of the given node to world space 
	*/
	MatrixTransform getWorldTransform(const string& node) const;
	
	/**
		returns the transform from world space to the space of the given node 
	*/
	MatrixTransform getInverseWorldTransform(const string& node) const;

private:
	RigidHierarchy hierarchy;
	Pose pose;
};

}

#endif