#ifndef _pc_pose_exaggeration_spatial_pose_exaggerator_h_
#define _pc_pose_exaggeration_spatial_pose_exaggerator_h_

/*
 *  SpatialPoseExaggerator.h
 *  PcPoseExaggeration
 *
 *  Created by Patrick Coleman on 7/27/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include <vector>
#include <map>
#include <string>

#include <PcMath/Vector3.h>
#include <PcMotion/MotionTransformation.h>
#include <PcMotion/SampledMotion.h>
#include <PcMotion/Pose.h>
#include <PcMotion/RigidHierarchy.h>

namespace PcPoseExaggeration {

using std::vector;
using std::map;
using std::string;
using PcMath::Vector3;
using PcMotion::MotionTransformation;
using PcMotion::SampledMotion;
using PcMotion::Pose;
using PcMotion::RigidHierarchy;

/**
	applies to a skeltal motion
*/
class SpatialPoseExaggerator: public MotionTransformation {

public:
	SpatialPoseExaggerator(const RigidHierarchy& rh, const vector<int>& extremePoseIndices,
						   float exaggeration = 1.0f, float bias = 0.5f,
						   bool rescalePoses = true, bool rescaleMotion = true);
	
	void apply(SampledMotion* originalMotion) const;
	vector<Pose> getExaggeratedPoses() const;  // returns exagerated poses from last application
	vector<Pose> getOriginalPoses() const;  // returns original poses from last application
	vector<Pose> getDifferencePoses() const;  // returns difference poses from last application
	
private:
	Pose subtractRoot(const Pose& p) const;
	Pose addRoot(const Pose& p) const;
	vector<Pose> exaggeratePoses(const vector<Pose>& originalPoses) const;
	
	map<const string, Vector3> getOffsets(const Pose& p) const;
	void correctOffsets(map<const string, Vector3>& offsets, const Pose& referencePose) const;
	void setOffsetsToPose(map<const string, Vector3>& offsets, Pose& pose,
					    const string& belowThisNode) const;
	void correctPoseLengths(Pose& pose, // corrects pose length with respect reference poses 
							const Pose& referencePose) const;
	
	void correctMotionLengths(SampledMotion* motion, const SampledMotion* referenceMotion) const;

	
private:
	//const SampledMotion* referenceSkeltalMotion;
	float exaggeration;
	float bias;
	RigidHierarchy rh;
	vector<int> extremePoseIndices;
	bool rescalePoses;
	bool rescaleMotion;
	
	mutable vector<Pose> exagPoses; // cached for exaggerated poses
	mutable vector<Pose> originalPoses; // cached for origina poses
	mutable vector<Pose> differencePoses; // cached for difference poses
};

}

#endif