#ifndef _pc_signal_signal_h_
#define _pc_signal_signal_h_

/*
 *  Signal.h
 *  PcSignal
 *
 *  Created by Patrick Coleman on 6/8/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include <math.h>
#include <vector>
#include <PcMath/functions.h>

namespace PcSignal {

using std::vector;
using PcMath::interpolate;

/**
	general signal class whose samples are of type T
	assumes uniform sampling
	interpolates between samples if evaluated at a float is float
*/
template <class T, T interpolate(const T& a, const T& b, double t)>
class Signal {

public:
	Signal() {};
	
	void append(const T& value);
	
	T& operator[](int i);
	const T& operator[](int i) const;
	T value(float t) const;
	int length() const;
	

private:
	vector<T> values;

};

template <class T, T interpolate(const T& a, const T& b, double t)>
void Signal<T, interpolate>::append(const T& value) {
	values.push_back(value);
}

template <class T, T interpolate(const T& a, const T& b, double t)>
T& Signal<T, interpolate>::operator[](int i) {
	return values[i];
}

template <class T, T interpolate(const T& a, const T& b, double t)>
const T& Signal<T, interpolate>::operator[](int i) const {
	return values[i];
}

template <class T, T interpolate(const T& a, const T& b, double t)>
T Signal<T, interpolate>::value(float t) const {
	
	int pi = (int)floor(t);
	if (pi < 0) {
		return values[0];
	} else if (pi >= (values.size() - 1)) {
		return values[values.size() - 1];
	} else {
		return interpolate(values[pi], values[pi + 1], t - pi);
	}
}

template <class T, T interpolate(const T& a, const T& b, double t)>
int Signal<T, interpolate>::length() const {
	return values.size();	
}

typedef Signal<float, PcMath::interpolate> FloatSignal;

}

#endif