/*
 *  TentFilter.cpp
 *  PcSignal
 *
 *  Created by Patrick Coleman on 8/27/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include "TentFilter.h"

namespace PcSignal {

FloatTentFilter::FloatTentFilter(const FloatSignal& signal, int filterWidth) :
	signal(signal), filterWidth(filterWidth) {
	
	if (this->filterWidth < 1) {
		this->filterWidth = 1;
	}
	if (this->filterWidth % 2 != 1) {
		this->filterWidth += 1;
	}

	this->halfWidth = (this->filterWidth - 1) / 2;
}

FloatSignal FloatTentFilter::output() const {
	FloatSignal res;
	
	if (filterWidth == 1) {
		for (int i = 0; i < signal.length(); i++) {
			res.append(signal[i]);
		}
	} else {
		for (int i = 0; i < signal.length(); i++) {
			int offset = i;
			if (halfWidth < i) {
				offset = halfWidth;
			}
			vector<float> mask = getWeightMask(i);
			float resp = 0.0;
			
			for (int j = -halfWidth; j <= halfWidth; j++) {
				int si = i + j;
				if ((si >= 0) and (si < signal.length())) {
					resp += mask[j + offset] * signal[si];
				}
			}
			res.append(resp);
		}
	}
	
	return res;
}

vector<float> FloatTentFilter::getWeightMask(int index) const {

	// construct basic tent mask
	vector<float> unclippedRes;
	unclippedRes.push_back(1.0);
	for (int i = 1; i <= halfWidth; i++) {
		float v = 1.0 - ((float)i) / (float)(halfWidth + 1);
		unclippedRes.push_back(v);
	}
		
	// clip off based on signal
	int startIndex = index - halfWidth;
	if (startIndex < 0) {
		startIndex = 0;
	}
	int endIndex = index + halfWidth;
	if (endIndex >= signal.length()) {
		endIndex = signal.length() - 1;
	}
	vector<float> res;
	float sum = 0.0;
	for (int i = startIndex; i <= endIndex; i++) {
		res.push_back(unclippedRes[i]);
		sum += unclippedRes[i];
	}
	
	//normalize
	float invDenom = 1.0 / sum;
	for (int i = 0; i < res.size(); i++) {
		res[i] = res[i] * invDenom;
	}
	
	return res;
}

}

