/*
 *  functions.cpp
 *  PcSignal
 *
 *  Created by Patrick Coleman on 7/26/09.
 *  Copyright 2009 patrick coleman. All rights reserved.
 *
 */

#include "functions.h"

namespace PcSignal {

vector<int> semilocalMaxima(vector<float> signal, int width, bool includeBounds) {

	vector<int> maxima;
	int halfWidth = width / 2;
	
	int startIndex = 1;
	int endIndex = signal.size() - 2;
	if (includeBounds) {
		startIndex = 0;
		endIndex += 1;
	}
	
	for (int i = startIndex; i <= endIndex; i++) {
		// get search bounds
		int startIndex = i - halfWidth;
		int endIndex = i + halfWidth;
		if (startIndex < 0) {
			startIndex = 0;
		}
		if (endIndex > (signal.size() - 1)) {
			endIndex = signal.size() - 1;
		}
		
		// determine maximality
		bool isMaxima = true;
		for (int j = startIndex; j <= endIndex; j++) {
			if ((signal[i] <= signal[j]) and (i != j)) {
				isMaxima = false;
				break;
			}
		}
		
		// append maxma if found 
		if (isMaxima) {
			maxima.push_back(i);
		}
	}
	
	/*
	for i in range(1, len(signal) - 1):
		
		# get search bounds
		startIndex = i - halfWidth
		endIndex = i + halfWidth
		if startIndex < 0:
			startIndex = 0
		if endIndex > (len(signal) - 1):
			endIndex = len(signal) - 1
		
		# determine maximality
		isMaxima = True
		for j in range(startIndex, endIndex + 1):
			if signal[i] <= signal[j] and i != j:
				isMaxima = False
				break
		
		# append maxma if found 
		if isMaxima:
			if returnType == 'Object':
				maxima.append(Maximum(i, signal[i]))
			else:
				maxima.append(i)
	*/
	
	return maxima;
}

vector<Vector3> getVelocitySignal(const vector<Point3>& signal, double dt) {

	vector<Vector3> velSig;
	
	int ns = signal.size();
	double invDt = 1.0 / dt;
	
	if (ns == 1) {
		velSig.push_back(Vector3());
	} else {
		if (ns > 1) {
			velSig.push_back(signal[1] - signal[0]);
		}
		
		for (int i = 1; i < ns - 1; i++) {
			velSig.push_back(invDt * (signal[i + 1] - signal[i - 1]));
		}
		
		if (ns > 2) {
			velSig.push_back(signal[ns - 1] - signal[ns - 2]);
		}
	}
	
	return velSig;
}

vector<double> getSpeedSquaredSignal(const vector<Point3>& signal, double dt) {
	
	vector<double> spdSig;
	
	int ns = signal.size();
	double invDt = 1.0 / dt;
	double inv2Dt = 0.5 * invDt;
	
	if (ns == 1) {
		spdSig.push_back(0.0);
	} else {
		if (ns > 1) {
			spdSig.push_back((invDt * (signal[1] - signal[0])).lengthSquared());
		}
		
		for (int i = 1; i < ns - 1; i++) {
			spdSig.push_back((inv2Dt * (signal[i + 1] - signal[i - 1])).lengthSquared());
		}
		
		if (ns > 2) {
			spdSig.push_back((invDt * (signal[ns - 1] - signal[ns - 2])).lengthSquared());
		}
	}
	
	return spdSig;
}

}